using System;
using System.Collections.Generic;
using System.Text;

namespace ReadDigitalInputs
{
    public class Msxexxxx : Msxe
    {
        /// <summary>
        /// System web reference
        /// </summary>
        private MSXExxxxWeb.MSXExxxx sysWeb;

        /// <summary>
        /// Number of Digital I/O channels
        /// </summary>
        private ulong digIOChannelNb = 0;

        /// <summary>
        /// Number of Digital I/O ports
        /// </summary>
        private ulong digIOPortNb = 0;

        /// <summary>
        /// Store the available digital I/O ports
        /// </summary>
        private List<DigitalIOPort> ports;

        /// <summary>
        /// Store the available digital I/Os
        /// </summary>
        private List<DigitalIO> ios;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="ip"></param>
        public Msxexxxx(string ip)
            : base(ip)
        {
            // Nothing specific
        }

        /// <summary>
        /// Override the connect method
        /// </summary>
        /// <param name="error"></param>
        /// <returns>
        /// 0:  success
        /// -1: wrong IP address
        /// -2: system error. Check the error variable for more informations
        /// -3: wrong system type
        /// -4: system not reachable
        /// -5: other error. Check the error variable for more informations
        /// </returns>
        public override int Connect(out string error)
        {
            ulong tmp = 0;
            ulong currentIoIndex = 0;

            // Call Connect of the base class
            // If successful, then the connectionState variable is set to 1
            int retval = base.Connect(out error);
            if (retval != 0)
                return retval;

            // Create the web-reference object
            this.sysWeb = new MSXExxxxWeb.MSXExxxx();
            try
            {
                this.sysWeb.Url = "http://" + this.IPAddress + ":5555";
                this.sysWeb.Timeout = 2000;
            }
            catch (Exception) { return -1; }

            // Check if the selected system has some digital I/O channel
            retval = this.DigitalIOGetNumberOfChannels(out tmp, out error);
            if (retval != 0)
            {
                this.connectionState = 0;
                return retval;
            }

            // Store number
            this.digIOChannelNb = tmp;

            // Any digital I/O channel?
            if (tmp == 0)
            {
                return -3;
            }

            // Check if the selected system has some digital I/O ports
            retval = this.DigitalIOGetNumberOfPorts(out tmp, out error);
            if (retval != 0)
            {
                this.connectionState = 0;
                return retval;
            }

            // Store number
            this.digIOPortNb = tmp;

            // Any digital I/O port?
            if (tmp == 0)
            {
                return -3;
            }

            // Construct the available ports and digital I/Os
            this.ports = new List<DigitalIOPort>();
            this.ios = new List<DigitalIO>();
            for (ulong i = 0; i < digIOPortNb; i++)
            {
                ulong availableInputs = 0;
                ulong availableOutputs = 0;
                ulong currentDirection = 0;
                ulong state = 0;

                // Get available direction
                retval = this.DigitalIOGetPortAvailableDirections(i, out availableInputs, out availableOutputs, out error);
                if (retval != 0)
                {
                    this.connectionState = 0;
                    return retval;
                }

                // Get current configuration
                retval = this.DigitalIOGetPortDirections(i, out currentDirection, out error);
                if (retval != 0)
                {
                    this.connectionState = 0;
                    return retval;
                }

                // Get the current state
                retval = this.DigitalIOReadPort(i, out state, out error);
                if (retval != 0)
                {
                    this.connectionState = 0;
                    return retval;
                }

                // Add the port
                DigitalIOPort port = new DigitalIOPort(i, availableInputs, availableOutputs);
                this.ports.Add(port);

                // Add the digital I/Os
                for (ulong j = 0; j < port.ChannelNumber; j++)
                {
                    bool inputPossible = false;
                    bool outputPossible = false;
                    ulong iostate = 0;

                    if (((availableInputs >> (int)j) & 1) == 1)
                    {
                        inputPossible = true;
                    }
                    if (((availableOutputs >> (int)j) & 1) == 1)
                    {
                        outputPossible = true;
                    }

                    // Compute the state
                    iostate = (state >> (int)j) & 1;

                    this.ios.Add(new DigitalIO(currentIoIndex,
                        i,
                        j,
                        inputPossible,
                        outputPossible,
                        (currentDirection >> (int)j) & 1,
                        iostate));
                    currentIoIndex++;
                }
            }

            return 0;
        }

        /// <summary>
        /// Update the internal state of the digital I/Os
        /// </summary>
        /// <param name="error"></param>
        /// <returns></returns>
        public int UpdateCurrentDigitalIOState(out string error)
        {
            int index = 0;
            // Default
            error = string.Empty;

            for (ulong i = 0; i < digIOPortNb; i++)
            {
                ulong state = 0;
                int retval = 0;
                DigitalIOPort port = this.GetDigitalIoPortInfo((int)i);

                // Get the current state
                retval = this.DigitalIOReadPort(i, out state, out error);
                if (retval != 0)
                {
                    return retval;
                }

                // Digital I/O
                for (ulong j = 0; j < port.ChannelNumber; j++)
                {
                    ulong iostate = 0;

                    // Compute the state
                    iostate = (state >> (int)j) & 1;

                    this.ios[index].CurrentState = iostate;
                    index++;
                }
            }
            return 0;
        }

        /// <summary>
        /// Get the number of digital I/O channels
        /// </summary>
        public ulong DigIOChannelNb
        {
            get
            {
                return this.digIOChannelNb;
            }
        }

        /// <summary>
        /// Get the number of digital I/O ports
        /// </summary>
        public ulong DigIOPortNb
        {
            get
            {
                return this.digIOPortNb;
            }
        }

        /// <summary>
        /// Get info concerning a digital I/O
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public DigitalIO GetDigitalIoInfo(int index)
        {
            return this.ios[index];
        }

        /// <summary>
        /// Get info concerning a digital output
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public DigitalIO GetDigitalOutputInfo(int index)
        {
            int curr = index;
            for (int i = 0; i < this.ios.Count; i++)
            {
                if (this.ios[i].CurrentConfiguration == 1)
                {
                    if (curr == 0)
                    {
                        return this.ios[i];
                    }
                    curr--;
                }
            }
            return null;
        }

        /// <summary>
        /// Get info concerning a digital I/O port
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public DigitalIOPort GetDigitalIoPortInfo(int index)
        {
            return this.ports[index];
        }

        #region "SOAP API"

        /// <summary>
        /// Reads the number of digital I/O channels
        /// </summary>
        /// <param name="channelNumber">Number of Digital I/O channels</param>
        /// <param name="error">Error message set if the return value is -2 or -5</param>
        /// <returns>
        /// 0:  success
        /// -1: you must call the Connect function before
        /// -2: system error. Check the error variable for more informations
        /// -3: the method is not implemented on the system. Please check the type of your system
        /// -4: the system is not reachable (not present on the network, or no SOAP server on it)
        /// -5: other error. Check the error variable for more informations
        /// </returns>
        public int DigitalIOGetNumberOfChannels(out ulong channelNumber, out string error)
        {
            // Default
            channelNumber = 0;
            error = string.Empty;

            // Check the connection state: Connect method should be called first
            if (this.connectionState != 1)
            {
                return -1;
            }

            // Call the SOAP function
            try
            {
                MSXExxxxWeb.DefaultResponse response = this.sysWeb.DigitalIOGetNumberOfChannels(0, out channelNumber);
                if (response.iReturnValue != 0)
                {
                    error = "The system returned an error when calling the MSXExxxx__DigitalIOGetNumberOfChannels function." + Environment.NewLine;
                    error += "Please check the documentation of this function for more information." + Environment.NewLine;
                    error += "(iReturnValue: " + response.iReturnValue.ToString() + ", syserrno: " + response.syserrno.ToString() + ")";
                    return -2;
                }
            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                return -3;
            }
            catch (System.Net.WebException)
            {
                return -4;
            }
            catch (Exception ex)
            {
                error = "Unhandled exception occured: " + ex.Message;
                return -5;
            }
            return 0;
        }

        /// <summary>
        /// Reads the number of digital I/O ports
        /// </summary>
        /// <param name="portNumber">Number of Digital I/O ports</param>
        /// <param name="error">Error message set if the return value is -2 or -5</param>
        /// <returns>
        /// 0:  success
        /// -1: you must call the Connect function before
        /// -2: system error. Check the error variable for more informations
        /// -3: the method is not implemented on the system. Please check the type of your system
        /// -4: the system is not reachable (not present on the network, or no SOAP server on it)
        /// -5: other error. Check the error variable for more informations
        /// </returns>
        public int DigitalIOGetNumberOfPorts(out ulong portNumber, out string error)
        {
            // Default
            portNumber = 0;
            error = string.Empty;

            // Check the connection state: Connect method should be called first
            if (this.connectionState != 1)
            {
                return -1;
            }

            // Call the SOAP function
            try
            {
                MSXExxxxWeb.DefaultResponse response = this.sysWeb.DigitalIOGetNumberOfPorts(0, out portNumber);
                if (response.iReturnValue != 0)
                {
                    error = "The system returned an error when calling the MSXExxxx__DigitalIOGetNumberOfPorts function." + Environment.NewLine;
                    error += "Please check the documentation of this function for more information." + Environment.NewLine;
                    error += "(iReturnValue: " + response.iReturnValue.ToString() + ", syserrno: " + response.syserrno.ToString() + ")";
                    return -2;
                }
            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                return -3;
            }
            catch (System.Net.WebException)
            {
                return -4;
            }
            catch (Exception ex)
            {
                error = "Unhandled exception occured: " + ex.Message;
                return -5;
            }
            return 0;
        }

        /// <summary>
        /// Get the available directions for the selected port
        /// </summary>
        /// <param name="port">The selected port</param>
        /// <param name="availableInput">Bit mask that represent the available inputs of the port</param>
        /// <param name="availableOutput">Bit mask that represent the available outputs of the port</param>
        /// <param name="error">Error message set if the return value is -2 or -5</param>
        /// <returns>
        /// 0:  success
        /// -1: you must call the Connect function before
        /// -2: system error. Check the error variable for more informations
        /// -3: the method is not implemented on the system. Please check the type of your system
        /// -4: the system is not reachable (not present on the network, or no SOAP server on it)
        /// -5: other error. Check the error variable for more informations
        /// </returns>
        public int DigitalIOGetPortAvailableDirections(ulong port, out ulong availableInput, out ulong availableOutput, out string error)
        {
            // Default
            availableInput = 0;
            availableOutput = 0;
            error = string.Empty;

            // Check the connection state: Connect method should be called first
            if (this.connectionState != 1)
            {
                return -1;
            }

            // Call the SOAP function
            try
            {
                MSXExxxxWeb.DefaultResponse response = this.sysWeb.DigitalIOGetPortAvailableDirections(port, 0, out availableInput, out availableOutput);
                if (response.iReturnValue != 0)
                {
                    error = "The system returned an error when calling the MSXExxxx__DigitalIOGetPortAvailableDirections function." + Environment.NewLine;
                    error += "Please check the documentation of this function for more information." + Environment.NewLine;
                    error += "(iReturnValue: " + response.iReturnValue.ToString() + ", syserrno: " + response.syserrno.ToString() + ")";
                    return -2;
                }
            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                return -3;
            }
            catch (System.Net.WebException)
            {
                return -4;
            }
            catch (Exception ex)
            {
                error = "Unhandled exception occured: " + ex.Message;
                return -5;
            }
            return 0;
        }

        /// <summary>
        /// Get the current direction of the selected port
        /// </summary>
        /// <param name="port">The selected port</param>
        /// <param name="direction">Bit mask that represent the configuration</param>
        /// <param name="error">Error message set if the return value is -2 or -5</param>
        /// <returns>
        /// 0:  success
        /// -1: you must call the Connect function before
        /// -2: system error. Check the error variable for more informations
        /// -3: the method is not implemented on the system. Please check the type of your system
        /// -4: the system is not reachable (not present on the network, or no SOAP server on it)
        /// -5: other error. Check the error variable for more informations
        /// </returns>
        public int DigitalIOGetPortDirections(ulong port, out ulong direction, out string error)
        {
            // Default
            direction = 0;
            error = string.Empty;

            // Check the connection state: Connect method should be called first
            if (this.connectionState != 1)
            {
                return -1;
            }

            // Call the SOAP function
            try
            {
                MSXExxxxWeb.DefaultResponse response = this.sysWeb.DigitalIOGetPortDirections(port, out direction);
                if (response.iReturnValue != 0)
                {
                    error = "The system returned an error when calling the MSXExxxx__DigitalIOGetPortDirections function." + Environment.NewLine;
                    error += "Please check the documentation of this function for more information." + Environment.NewLine;
                    error += "(iReturnValue: " + response.iReturnValue.ToString() + ", syserrno: " + response.syserrno.ToString() + ")";
                    return -2;
                }
            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                return -3;
            }
            catch (System.Net.WebException)
            {
                return -4;
            }
            catch (Exception ex)
            {
                error = "Unhandled exception occured: " + ex.Message;
                return -5;
            }
            return 0;
        }

        /// <summary>
        /// Set the current direction of the selected port
        /// </summary>
        /// <param name="port">The selected port</param>
        /// <param name="direction">Bit mask that represent the configuration</param>
        /// <param name="error">Error message set if the return value is -2 or -5</param>
        /// <returns>
        /// 0:  success
        /// -1: you must call the Connect function before
        /// -2: system error. Check the error variable for more informations
        /// -3: the method is not implemented on the system. Please check the type of your system
        /// -4: the system is not reachable (not present on the network, or no SOAP server on it)
        /// -5: other error. Check the error variable for more informations
        /// </returns>
        public int DigitalIOSetPortDirections(ulong port, ulong direction, out string error)
        {
            // Default
            error = string.Empty;

            // Check the connection state: Connect method should be called first
            if (this.connectionState != 1)
            {
                return -1;
            }

            // Call the SOAP function
            try
            {
                int syserrno = 0;
                int retval = this.sysWeb.DigitalIOSetPortDirections(port, direction, 0, out syserrno);
                if (retval != 0)
                {
                    error = "The system returned an error when calling the MSXExxxx__DigitalIOSetPortDirections function." + Environment.NewLine;
                    error += "Please check the documentation of this function for more information." + Environment.NewLine;
                    error += "(iReturnValue: " + retval.ToString() + ", syserrno: " + syserrno.ToString() + ")";
                    return -2;
                }
            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                return -3;
            }
            catch (System.Net.WebException)
            {
                return -4;
            }
            catch (Exception ex)
            {
                error = "Unhandled exception occured: " + ex.Message;
                return -5;
            }
            return 0;
        }

        /// <summary>
        /// Set the selected digital output channel to on or off.
        /// </summary>
        /// <param name="channel">	Selected digital I/O channel</param>
        /// <param name="state">0 or 1</param>
        /// <param name="error">Error message set if the return value is -2 or -5</param>
        /// <returns>
        /// 0:  success
        /// -1: you must call the Connect function before
        /// -2: system error. Check the error variable for more informations
        /// -3: the method is not implemented on the system. Please check the type of your system
        /// -4: the system is not reachable (not present on the network, or no SOAP server on it)
        /// -5: other error. Check the error variable for more informations
        /// </returns>
        public int DigitalIOWriteChannel(ulong channel, ulong state, out string error)
        {
            // Default
            error = string.Empty;

            // Check the connection state: Connect method should be called first
            if (this.connectionState != 1)
            {
                return -1;
            }

            // Call the SOAP function
            try
            {
                int syserrno = 0;
                int retval = this.sysWeb.DigitalIOWriteChannel(channel, state, 0, out syserrno);
                if (retval != 0)
                {
                    error = "The system returned an error when calling the MSXExxxx__DigitalIOWriteChannel function." + Environment.NewLine;
                    error += "Please check the documentation of this function for more information." + Environment.NewLine;
                    error += "(iReturnValue: " + retval.ToString() + ", syserrno: " + syserrno.ToString() + ")";
                    return -2;
                }
            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                return -3;
            }
            catch (System.Net.WebException)
            {
                return -4;
            }
            catch (Exception ex)
            {
                error = "Unhandled exception occured: " + ex.Message;
                return -5;
            }
            return 0;
        }

        /// <summary>
        /// Read the selected digital I/O port
        /// </summary>
        /// <param name="port">The selected port</param>
        /// <param name="state">Digital I/O state. Each bit indicates the state of one digital I/O channel.</param>
        /// <param name="error">Error message set if the return value is -2 or -5</param>
        /// <returns>
        /// 0:  success
        /// -1: you must call the Connect function before
        /// -2: system error. Check the error variable for more informations
        /// -3: the method is not implemented on the system. Please check the type of your system
        /// -4: the system is not reachable (not present on the network, or no SOAP server on it)
        /// -5: other error. Check the error variable for more informations
        /// </returns>
        public int DigitalIOReadPort(ulong port, out ulong state, out string error)
        {
            // Default
            state = 0;
            error = string.Empty;

            // Check the connection state: Connect method should be called first
            if (this.connectionState != 1)
            {
                return -1;
            }

            // Call the SOAP function
            try
            {
                MSXExxxxWeb.DefaultResponse response = this.sysWeb.DigitalIOReadPort(port, 0, out state);
                if (response.iReturnValue != 0)
                {
                    error = "The system returned an error when calling the MSXExxxx__DigitalIOReadPort function." + Environment.NewLine;
                    error += "Please check the documentation of this function for more information." + Environment.NewLine;
                    error += "(iReturnValue: " + response.iReturnValue.ToString() + ", syserrno: " + response.syserrno.ToString() + ")";
                    return -2;
                }
            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                return -3;
            }
            catch (System.Net.WebException)
            {
                return -4;
            }
            catch (Exception ex)
            {
                error = "Unhandled exception occured: " + ex.Message;
                return -5;
            }
            return 0;
        }

        #endregion
    }
}

