using System;
using System.Collections.Generic;
using System.Text;

namespace CompareLogic
{
    public class Msxexxxx : Msxe
    {
        /// <summary>
        /// System web reference
        /// </summary>
        private MSXExxxxWeb.MSXExxxx sysWeb;

        /// <summary>
        /// Number of Sinus Cosinus channels
        /// </summary>
        private ulong sinCosChannelNb = 0;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="ip"></param>
        public Msxexxxx(string ip)
            : base(ip)
        {
            // Nothing specific
        }

        /// <summary>
        /// Override the connect method
        /// </summary>
        /// <param name="error"></param>
        /// <returns>
        /// 0:  success
        /// -1: wrong IP address
        /// -2: system error. Check the error variable for more informations
        /// -3: wrong system type
        /// -4: system not reachable
        /// -5: other error. Check the error variable for more informations
        /// </returns>
        public override int Connect(out string error)
        {
            ulong tmp = 0;
            // Call Connect of the base class
            // If successful, then the connectionState variable is set to 1
            int retval = base.Connect(out error);
            if (retval != 0)
                return retval;

            // Create the web-reference object
            this.sysWeb = new MSXExxxxWeb.MSXExxxx();
            try
            {
                this.sysWeb.Url = "http://" + this.IPAddress + ":5555";
                this.sysWeb.Timeout = 2000;
            }
            catch (Exception) { return -1; }

            // Check if the selected system has some Sinus Cosinus channel
            retval = this.SinCosGetNumberOfChannels(out tmp, out error);
            if (retval != 0)
            {
                this.connectionState = 0;
                return retval;
            }

            // Store number
            this.sinCosChannelNb = tmp;

            // Any Sinus Cosinus channel?
            if (tmp == 0)
            {
                return -3;
            }

            return 0;
        }

        /// <summary>
        /// Get the number of sin cos channels
        /// </summary>
        public ulong SinCosChannelNb
        {
            get
            {
                return this.sinCosChannelNb;
            }
        }

        #region "SOAP API"

        /// <summary>
        /// Reads the number of Sinus Cosinus channels
        /// </summary>
        /// <param name="channelNumber">Number of Sinus Cosinus channels</param>
        /// <param name="error">Error message set if the return value is -2 or -5</param>
        /// <returns>
        /// 0:  success
        /// -1: you must call the Connect function before
        /// -2: system error. Check the error variable for more informations
        /// -3: the method is not implemented on the system. Please check the type of your system
        /// -4: the system is not reachable (not present on the network, or no SOAP server on it)
        /// -5: other error. Check the error variable for more informations
        /// </returns>
        public int SinCosGetNumberOfChannels(out ulong channelNumber, out string error)
        {
            // Default
            channelNumber = 0;
            error = string.Empty;

            // Check the connection state: Connect method should be called first
            if (this.connectionState != 1)
            {
                return -1;
            }

            // Call the SOAP function
            try
            {
                MSXExxxxWeb.DefaultResponse response = this.sysWeb.SinCosGetNumberOfChannels(0, out channelNumber);
                if (response.iReturnValue != 0)
                {
                    error = "The system returned an error when calling the MSXExxxx__SinCosGetNumberOfChannels function." + Environment.NewLine;
                    error += "Please check the documentation of this function for more information." + Environment.NewLine;
                    error += "(iReturnValue: " + response.iReturnValue.ToString() + ", syserrno: " + response.syserrno.ToString() + ")";
                    return -2;
                }
            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                return -3;
            }
            catch (System.Net.WebException)
            {
                return -4;
            }
            catch (Exception ex)
            {
                error = "Unhandled exception occured: " + ex.Message;
                return -5;
            }
            return 0;
        }

        /// <summary>
        /// Get the current Sinus Cosinus configuration
        /// </summary>
        /// <param name="channel">Channel selection</param>
        /// <param name="initialisationState">Initialisation state of the Sinus Cosinus channel</param>
        /// <param name="signalPeriod">Signal period (m/period)</param>
        /// <param name="resolution">Resolution (steps/period)</param>
        /// <param name="indexState"></param>
        /// <param name="indexEdge"></param>
        /// <param name="indexAction"></param>
        /// <param name="compareLogicState"></param>
        /// <param name="compareLogicValue"></param>
        /// <param name="cmopareLogicValueFormat"></param>
        /// <param name="compareLogicMode"></param>
        /// <param name="compareLogicSynchro"></param>
        /// <param name="error">Error message set if the return value is -2 or -5</param>
        /// <returns>
        /// 0:  success
        /// -1: you must call the Connect function before
        /// -2: system error. Check the error variable for more informations
        /// -3: the method is not implemented on the system. Please check the type of your system
        /// -4: the system is not reachable (not present on the network, or no SOAP server on it)
        /// -5: other error. Check the error variable for more informations
        /// </returns>
        public int SinCosGetConfiguration(ulong channel,
            out ulong initialisationState,
            out double signalPeriod,
            out ulong resolution,
            out ulong indexState,
            out ulong indexEdge,
            out ulong indexAction,
            out ulong compareLogicState,
            out double compareLogicValue,
            out ulong compareLogicValueFormat,
            out ulong compareLogicMode,
            out ulong compareLogicSynchro,
            out string error)
        {
            // Default
            ulong info01 = 0;
            ulong info02 = 0;
            error = string.Empty;
            initialisationState = 0;
            signalPeriod = 0;
            resolution = 0;
            indexState = 0;
            indexEdge = 0;
            indexAction = 0;
            compareLogicState = 0;
            compareLogicValue = 0;
            compareLogicValueFormat = 0;
            compareLogicMode = 0;
            compareLogicSynchro = 0;

            // Check the connection state: Connect method should be called first
            if (this.connectionState != 1)
            {
                return -1;
            }

            // Call the SOAP function
            try
            {
                MSXExxxxWeb.DefaultResponse response = this.sysWeb.SinCosGetConfiguration(channel, 0, 0,
                    out initialisationState,
                    out signalPeriod,
                    out resolution,
                    out indexState, out indexEdge, out indexAction,
                    out compareLogicState, out compareLogicValue, out compareLogicValueFormat,
                    out compareLogicMode, out compareLogicSynchro,
                    out info01, out info02);
                if (response.iReturnValue != 0)
                {
                    error = "The system returned an error when calling the MSXExxxx__SinCosGetConfiguration function." + Environment.NewLine;
                    error += "Please check the documentation of this function for more information." + Environment.NewLine;
                    error += "(iReturnValue: " + response.iReturnValue.ToString() + ", syserrno: " + response.syserrno.ToString() + ")";
                    return -2;
                }
            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                return -3;
            }
            catch (System.Net.WebException)
            {
                return -4;
            }
            catch (Exception ex)
            {
                error = "Unhandled exception occured: " + ex.Message;
                return -5;
            }
            return 0;
        }

        /// <summary>
        /// Initialise and enable a Sinus Cosinus comparelogic
        /// </summary>
        /// <param name="channel">Channel selection</param>
        /// <param name="value">Compare value</param>
        /// <param name="valueFormat">Compare value format (0 raw, 1 standardised)</param>
        /// <param name="mode">Compare mode</param>
        /// <param name="synchro">Generate a synchro trigger if set to 1</param>
        /// <param name="error">Error message set if the return value is -2 or -5</param>
        /// <returns>
        /// 0:  success
        /// -1: you must call the Connect function before
        /// -2: system error. Check the error variable for more informations
        /// -3: the method is not implemented on the system. Please check the type of your system
        /// -4: the system is not reachable (not present on the network, or no SOAP server on it)
        /// -5: other error. Check the error variable for more informations
        /// </returns>
        public int SinCosInitCompareLogic(ulong channel, double value, ulong valueFormat, ulong mode, ulong synchro, out string error)
        {
            // Default
            error = string.Empty;

            // Check the connection state: Connect method should be called first
            if (this.connectionState != 1)
            {
                return -1;
            }

            // Call the SOAP function
            try
            {
                int syserrno = 0;
                int retval = this.sysWeb.SinCosInitAndEnableCompareLogic(channel, value, valueFormat, mode, synchro, 0, 0, out syserrno);
                if (retval != 0)
                {
                    error = "The system returned an error when calling the MSXExxxx__SinCosInitAndEnableCompareLogic function." + Environment.NewLine;
                    error += "Please check the documentation of this function for more information." + Environment.NewLine;
                    error += "(iReturnValue: " + retval.ToString() + ", syserrno: " + syserrno.ToString() + ")";
                    return -2;
                }
            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                return -3;
            }
            catch (System.Net.WebException)
            {
                return -4;
            }
            catch (Exception ex)
            {
                error = "Unhandled exception occured: " + ex.Message;
                return -5;
            }
            return 0;
        }

        /// <summary>
        /// Release the Sinus Cosinus compare logic
        /// </summary>
        /// <param name="channel">Channel selection</param>
        /// <param name="error">Error message set if the return value is -2 or -5</param>
        /// <returns>
        /// 0:  success
        /// -1: you must call the Connect function before
        /// -2: system error. Check the error variable for more informations
        /// -3: the method is not implemented on the system. Please check the type of your system
        /// -4: the system is not reachable (not present on the network, or no SOAP server on it)
        /// -5: other error. Check the error variable for more informations
        /// </returns>
        public int SinCosReleaseCompareLogic(ulong channel, out string error)
        {
            // Default
            error = string.Empty;

            // Check the connection state: Connect method should be called first
            if (this.connectionState != 1)
            {
                return -1;
            }

            // Call the SOAP function
            try
            {
                int syserrno = 0;
                int retval = this.sysWeb.SinCosDisableAndReleaseCompareLogic(channel, 0, 0, out syserrno);
                if (retval != 0)
                {
                    error = "The system returned an error when calling the MSXExxxx__SinCosDisableAndReleaseCompareLogic function." + Environment.NewLine;
                    error += "Please check the documentation of this function for more information." + Environment.NewLine;
                    error += "(iReturnValue: " + retval.ToString() + ", syserrno: " + syserrno.ToString() + ")";
                    return -2;
                }
            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                return -3;
            }
            catch (System.Net.WebException)
            {
                return -4;
            }
            catch (Exception ex)
            {
                error = "Unhandled exception occured: " + ex.Message;
                return -5;
            }
            return 0;
        }

        #endregion
    }
}
