using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace TransducerCalibrationData
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// Represents the MSX-E system
        /// </summary>
        private MSXExxxx mx;

        public MainForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Occurs the first time the form is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // Load the parameters
            this.ipTextBox.Text = Properties.Settings.Default.ip;
        }

        /// <summary>
        /// Occurs when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Save the parameters
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Click on initialise
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void initButton_Click(object sender, EventArgs e)
        {
            string error = string.Empty;
            ulong channelNb = 0;

            // Construct MSX-E and get number of channels
            this.mx = new MSXExxxx(this.ipTextBox.Text);

            // Get the number of channels
            if (this.mx.TransducerGetNumberOfChannels(ref channelNb, ref error) != 0)
            {
                this.AddLog(error);
                return;
            }

            // Populate the channel selection combobox
            string[] src = new string[channelNb];
            this.channelComboBox.DataSource = null;
            for (ulong i = 0; i < channelNb; i++)
            {
                src[i] = i.ToString();
            }
            this.channelComboBox.DataSource = src;
        }

        /// <summary>
        /// Add a log
        /// </summary>
        /// <param name="log"></param>
        private void AddLog(String log)
        {
            this.logTextBox.Text += DateTime.Now.ToString() + "\t" + log + Environment.NewLine;
            this.logTextBox.SelectionStart = this.logTextBox.Text.Length;
            this.logTextBox.ScrollToCaret();
        }

        /// <summary>
        /// Selected index changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void channelComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (channelComboBox.SelectedIndex < 0)
                return;

            string error = string.Empty;
            List<TransducerCalibration> calibrationData = null;

            // Get the calibration data
            if (this.mx.TransducerGetChannelTransducersCalibrationDataList((ulong)channelComboBox.SelectedIndex,
                ref calibrationData,
                ref error) != 0)
            {
                this.AddLog(error);
                return;
            }

            this.calibrationTextBox.Text = string.Empty;

            // Dump
            foreach (TransducerCalibration t in calibrationData)
            {
                this.calibrationTextBox.Text += "Transducer index: " + t.index.ToString() + Environment.NewLine;
                this.calibrationTextBox.Text += "\t Used: " + t.used.ToString() + Environment.NewLine;
                this.calibrationTextBox.Text += "\t Calibrated: " + t.calibrated.ToString() + Environment.NewLine;
                this.calibrationTextBox.Text += "\t Secondary Gain Coarse: " + t.secGainCoarse.ToString() + Environment.NewLine;
                this.calibrationTextBox.Text += "\t Secondary Gain Fine: " + t.secGainFine.ToString() + Environment.NewLine;
                this.calibrationTextBox.Text += "\t Feedback Gain Coarse: " + t.feedbackGainCoarse.ToString() + Environment.NewLine;
                this.calibrationTextBox.Text += "\t Feedback Gain Fine: " + t.feedbackGainFine.ToString() + Environment.NewLine;
                this.calibrationTextBox.Text += "\t Feedback Offset: " + t.feedbackOffset.ToString() + Environment.NewLine;
                this.calibrationTextBox.Text += "\t Correction factor left: " + t.correctionFactorLeft.ToString() + Environment.NewLine;
                this.calibrationTextBox.Text += "\t Correction factor right: " + t.correctionFactorRight.ToString() + Environment.NewLine;
            }

            if (calibrationData.Count == 0)
            {
                this.calibrationTextBox.Text = "No calibration for the selected channel.";
            }
        }
    }
}